//	Enemy.cpp - The basic enemy class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "ColourHelpers.h"
#include "Random.h"
#include "Enemy.h"
#include "Shard.h"

//------------------------------------------------------------------------------
Enemy::Enemy(const TwoFloats& pos):
position(pos),
divertCount(0.0f),
fadeIn(0.0f),
dead(false),
blue(false)
{
	ThreeFloats hslCol;

	speed = 0.85f + (Random::getGlobal().getRandomFloat() * 0.3f);

	hslCol.x = Random::getGlobal().getRandomFloat();
	hslCol.y = 0.75f;
	hslCol.z = 0.75f;
	shards.push_back(shared_ptr<Shard>(new Shard(L"EnemyTri0",
									   ColourHelpers::hslToRgb(hslCol),//ThreeFloats(146.0f/255.0f, 1.0f, 175.0f/255.0f),
									   TwoFloats(9.0f))));
	shards.back()->setPosition(position-32.0f);
	hslCol.x += (1.0f/6.0f);
	if(hslCol.x > 1.0f)
		hslCol.x -= 1.0f;
	shards.push_back(shared_ptr<Shard>(new Shard(L"EnemyTri1",
												 ColourHelpers::hslToRgb(hslCol),//ThreeFloats(1.0f, 1.0f, 146.0f/255.0f),
												 TwoFloats(28.0f, 10.0f))));
	shards.back()->setPosition(position-32.0f);
	hslCol.x += (1.0f/6.0f);
	if(hslCol.x > 1.0f)
		hslCol.x -= 1.0f;
	shards.push_back(shared_ptr<Shard>(new Shard(L"EnemyTri2",
												 ColourHelpers::hslToRgb(hslCol),//ThreeFloats(146.0f/255.0f, 1.0f, 224.0f/255.0f),
												 TwoFloats(24.0f, 18.0f))));
	shards.back()->setPosition(position-32.0f);
	hslCol.x += (1.0f/6.0f);
	if(hslCol.x > 1.0f)
		hslCol.x -= 1.0f;
	shards.push_back(shared_ptr<Shard>(new Shard(L"EnemyTri3",
												 ColourHelpers::hslToRgb(hslCol),//ThreeFloats(146.0f/255.0f, 147.0f/255.0f, 1.0f),
												 TwoFloats(13.0f, 23.0f))));
	shards.back()->setPosition(position-32.0f);
	hslCol.x += (1.0f/6.0f);
	if(hslCol.x > 1.0f)
		hslCol.x -= 1.0f;
	shards.push_back(shared_ptr<Shard>(new Shard(L"EnemyTri4",
												 ColourHelpers::hslToRgb(hslCol),//ThreeFloats(1.0f, 146.0f/255.0f, 146.0f/255.0f),
												 TwoFloats(18.0f, 22.0f))));
	shards.back()->setPosition(position-32.0f);
}

//------------------------------------------------------------------------------
Enemy::Enemy(const Enemy& other):
position(other.position),
speed(other.speed),
divertCount(0.0f),
bulletCount(60),
fadeIn(0.0f),
dead(false),
blue(false)
{
	unsigned int i;
	const std::vector<shared_ptr<Shard> >& otherShards = other.getShards();

	for(i=0;i<otherShards.size();++i)
		shards.push_back(shared_ptr<Shard>(new Shard(*(otherShards[i]))));
}

//------------------------------------------------------------------------------
Enemy::~Enemy()
{
	unsigned int i;

	for(i=0;i<shards.size();++i)
		shards[i]->setDirection((shards[i]->getPosition() - position).normal());
}

//------------------------------------------------------------------------------
bool Enemy::update(const TwoFloats& playerPos)
{
	TwoFloats dir;
	unsigned int i;
	bool retval = false;

	if(fadeIn >= 1.0f)
	{
		dir = (playerPos-position).normal();

		if(divertCount > 0.0f)
		{
			dir *= 1.0f-divertCount;
			dir += divertDir * divertCount;

			divertCount -= 0.05f;
		}

		position += dir * 2.0f * speed;

		for(i=0;i<shards.size();++i)
			shards[i]->setPosition(position-32.0f);
	}
	else
	{
		fadeIn += 0.01f;
		if(fadeIn > 1.0f)
			fadeIn = 1.0f;
	}

	if(shards.size() > 5)
	{
		--bulletCount;
		if(!bulletCount)
		{
			int tempint = shards.size() - 5;

			if(tempint > 9)
				tempint = 9;
			tempint = 10 - tempint;

			bulletCount = (30*(tempint/2)) + Random::getGlobal().getRandomInt(30 * tempint);

			retval = true;
		}
	}

	return retval;
}

//------------------------------------------------------------------------------
void Enemy::draw(shared_ptr<Drawer>& d)
{
	if((position > TwoFloats(-64.0f)) &&
	   (position < (d->getDrawingAreaSize()+64.0f)))
	{
		unsigned int i;

		for(i=0;i<shards.size();++i)
			shards[i]->draw(d, fadeIn);
	}
}

//------------------------------------------------------------------------------
void Enemy::collided(const std::wstring& other, const TwoFloats& otherPos)
{
	if((other == L"Player") || (other == L"Mine") || (other == L"PlayerBullet"))
	{
		if(shards.size() > 5)
			shards.erase(shards.begin()+(shards.size()-1));
		else
			dead = true;
	}
	else if((other == L"Enemy") && (divertCount <= 0.0f))
	{
		divertDir = (position - otherPos).normal();
		divertCount = 1.0f;
	}
}

//------------------------------------------------------------------------------
void Enemy::incPosition(const TwoFloats& pos)
{
	unsigned int i;

	position += pos;

	for(i=0;i<shards.size();++i)
		shards[i]->setPosition(position-32.0f);
}

//------------------------------------------------------------------------------
void Enemy::addShard(shared_ptr<Shard> shard)
{
	shard->reset(shard->getPosition() - (position-32.0f));
	shards.push_back(shard);
	speed *= 0.75f;
}

//------------------------------------------------------------------------------
void Enemy::setFlash()
{
	unsigned int i;

	for(i=0;i<shards.size();++i)
		shards[i]->setFlash();
}

//------------------------------------------------------------------------------
void Enemy::setBlue()
{
	unsigned int i;

	for(i=0;i<shards.size();++i)
		shards[i]->setBlue();

	blue = true;
}

//------------------------------------------------------------------------------
void Enemy::setDead()
{
	dead = true;
}
