//	Enemy.h - The basic enemy class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ENEMY_H_
#define ENEMY_H_

#include "c0xHeaders.h"
#include "Collidable.h"

#include <vector>

class Drawer;
class Shard;

///	The basic enemy class.
class Enemy : public Collidable
{
  public:
	///	Constructor,
	Enemy(const TwoFloats& pos);
	///	Copy Constructor.
	Enemy(const Enemy& other);
	///	Destructor.
	~Enemy();

	///	Moves the enemy towards the player.
	/*!
		\return True if the enemy has fired a bullet.
	 */
	bool update(const TwoFloats& playerPos);
	///	Draws the enemy.
	void draw(shared_ptr<Drawer>& d);

	///	Returns the enemy's position.
	const TwoFloats& getPosition() const {return position;};
	///	The enemy's radius.
	float getRadius() const {return 24.0f;};
	///	The enemy's type name for collision purposes.
	const std::wstring getName() const {return L"Enemy";};
	///	Called when the enemy collides with another actor.
	void collided(const std::wstring& other, const TwoFloats& otherPos);
	///	Returns true while the enemy is fading in.
	bool doNotCount() const {return fadeIn < 1.0f;};

	///	Used to set the enemy's starting position on paste.
	void incPosition(const TwoFloats& pos);

	///	Adds a shard to the enemy.
	void addShard(shared_ptr<Shard> shard);

	///	Flashes the enemy when it's been copied.
	void setFlash();
	///	Marks the enemy as selected.
	void setBlue();
	///	Kills the enemy.
	void setDead();

	///	Returns true if the enemy is selected.
	bool getBlue() const {return blue;};
	///	Returns true if the enemy is dead.
	bool getDead() const {return dead;};
	///	Returns the vector of this enemy's shards.
	const std::vector<shared_ptr<Shard> >& getShards() const {return shards;};
  private:
	///	The enemy's current position.
	TwoFloats position;
	///	The enemy's speed.
	float speed;

	///	Used to divert the enemy when it collides with another.
	TwoFloats divertDir;
	///	Used to divert the enemy when it collides with another.
	float divertCount;

	///	The enemy's shards.
	std::vector<shared_ptr<Shard> > shards;

	///	Used to determine when to fire a bullet.
	int bulletCount;

	///	Used to fade the enemy in at it's birth.
	float fadeIn;
	///	Whether or not the enemy is dead.
	bool dead;
	///	True if the enemy is selected.
	bool blue;
};

#endif
