//	GamePlayState.cpp - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PastarenaGlobalData.h"
#include "PastarenaSounder.h"
#include "drawing/Drawer.h"
#include "GamePlayState.h"
#include "PlayerBullet.h"
#include "EnemyBullet.h"
#include "MiscHelper.h"
#include "Random.h"
#include "Player.h"
#include "Pickup.h"
#include "Enemy.h"
#include "Shard.h"
#include "Mine.h"

#include <sstream>

using std::wstring;
using std::vector;

//------------------------------------------------------------------------------
GameStateCreator<GamePlayState> GamePlayState::creator;

//------------------------------------------------------------------------------
GamePlayState::GamePlayState(Application& app, SounderPtr sound):
GameState(app, sound),
backTween0(backPos[0], backPos[0], 0.001f),
backTween1(backPos[1], backPos[1], 0.0015f),
backSizeTween0(backSize[0], backSize[0], 0.002f),
backSizeTween1(backSize[1], backSize[1], 0.0011f),
backColTween(backColour, backColour, 0.001f),
player(new Player()),
playerBulletCount(8),
clipboardCount(1),
clipboardCopyLast(true),
clipboardPitch(1.0f),
pickupCount(360),
currentPickup(-1),
pickupClipboardCount(0),
shakeIndex(0.0f),
shakeVal(0.0f),
points(1.0f),
sounder(dynamic_pointer_cast<PastarenaSounder>(sound))
{
	int i;
	float angle;
	TwoFloats radius;
	TwoFloats tempPos;
	const int NumMines = 8;
	const int NumEnemies = 16;

	globalData = dynamic_pointer_cast<PastarenaGlobalData>(getGlobalData());

	//Setup the background.
	tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
	tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
	backTween0.reset(tempPos);

	tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
	tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
	backPos[1] = TwoFloats(0.33f, 0.1f);
	backTween1.reset(tempPos);

	backSize[0] = 1.0f;
	backSize[1] = 1.0f;
	backSizeTween0.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));
	backSizeTween1.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));

	backColour = getBackColour();
	backColTween.reset(getBackColour());

	collidables.push_back(weak_ptr<Collidable>(player));

	//Setup the enemies.
	for(i=0;i<NumEnemies;++i)
	{
		angle = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
		radius.x = 256.0f + (Random::getGlobal().getRandomFloat() * ((getDrawingAreaSize().x*0.5f)-256.0f));
		radius.y = 256.0f + (Random::getGlobal().getRandomFloat() * ((getDrawingAreaSize().y*0.5f)-256.0f));

		tempPos.x = sinf(angle);
		tempPos.y = cosf(angle);
		tempPos *= radius;
		tempPos += (getDrawingAreaSize()*0.5f);

		enemies.push_back(shared_ptr<Enemy>(new Enemy(tempPos)));
		collidables.push_back(weak_ptr<Collidable>(enemies.back()));

		undoClipboard.enemies.push_back(shared_ptr<Enemy>(new Enemy(*(enemies[i]))));
	}
	//Setup the mines.
	for(i=0;i<NumMines;++i)
	{
		angle = Random::getGlobal().getRandomFloat() * 2.0f * NIALL_PI;
		radius.x = 256.0f + (Random::getGlobal().getRandomFloat() * ((getDrawingAreaSize().x*0.5f)-256.0f));
		radius.y = 256.0f + (Random::getGlobal().getRandomFloat() * ((getDrawingAreaSize().y*0.5f)-256.0f));

		tempPos.x = sinf(angle);
		tempPos.y = cosf(angle);
		tempPos *= radius;
		tempPos += (getDrawingAreaSize()*0.5f);

		mines.push_back(shared_ptr<Mine>(new Mine(tempPos)));
		collidables.push_back(weak_ptr<Collidable>(mines.back()));

		undoClipboard.mines.push_back(shared_ptr<Mine>(new Mine(*(mines[i]))));
	}

	undoClipboard.playerPos = getDrawingAreaSize() * 0.5f;

	setMousePos(getDrawingAreaSize()*0.5f);

	sounder->loadPitches();
	globalData->getGameCounter().reset();
}

//------------------------------------------------------------------------------
GamePlayState::~GamePlayState()
{

}

//------------------------------------------------------------------------------
void GamePlayState::update()
{
	int i, j;
	float tempf;
	TwoFloats tempPos;
	const TwoFloats origin;
	const TwoFloats& playerPos = player->getPosition();

	if(!player->getDead())
	{
		//Do a clipboard action if necessary.
		--clipboardCount;
		if(!clipboardCount)
		{
			if(!mainClipboard.enemies.size())
				mainClipboardCopy();
			else if(enemies.size() > 96)
			{
				enemies.clear();
				mines.clear();
				undoClipboardPaste();
				clipboardCopyLast = false;
			}
			else if(!enemies.size() || (Random::getGlobal().getRandomFloat() > 0.95f))
			{
				enemies.clear();
				mines.clear();
				undoClipboardPaste();
				clipboardCopyLast = false;
			}
			else
			{
				int tempint = Random::getGlobal().getRandomInt(3);

				if(tempint > 2)
					tempint = 2;

				switch(tempint)
				{
					case 0:
						if(!clipboardCopyLast)
							mainClipboardCopy();
						else
							mainClipboardPaste();
						clipboardCopyLast = !clipboardCopyLast;
						break;
					case 1:
						mainClipboardPaste();
						clipboardCopyLast = false;
						break;
					case 2:
						if(!clipboardCopyLast && (enemies.size() > 32))
							mainClipboardCut();
						else
							mainClipboardPaste();
						clipboardCopyLast = !clipboardCopyLast;
						break;
					case 3:
						mines.clear();
						enemies.clear();
						undoClipboardPaste();
						clipboardCopyLast = false;
				}
			}

			clipboardCount = 120 + (Random::getGlobal().getRandomInt(120));
		}

		//Add a new pickup if necessary.
		--pickupCount;
		if(!pickupCount)
		{
			tempPos.x = 64.0f + (Random::getGlobal().getRandomFloat() * (getDrawingAreaSize().x-128.0f));
			tempPos.y = 64.0f + (Random::getGlobal().getRandomFloat() * (getDrawingAreaSize().y-128.0f));

			pickups.push_back(shared_ptr<Pickup>(new Pickup(tempPos)));
			collidables.push_back(weak_ptr<Collidable>(pickups.back()));

			pickupCount = 360 + (Random::getGlobal().getRandomInt(360));
		}

		//Add new player bullets if necessary.
		--playerBulletCount;
		if(!playerBulletCount)
		{
			playerBullets.push_back(shared_ptr<PlayerBullet>(new PlayerBullet(player->getPosition(),
																			  player->getDirection())));
			collidables.push_back(weak_ptr<Collidable>(playerBullets.back()));
			tempf = playerBullets.back()->getAngle();
			tempf += NIALL_PI;
			if(tempf > (2.0f*NIALL_PI))
				tempf -= (2.0f*NIALL_PI);
			else if(tempf < 0.0f)
				tempf += NIALL_PI;
			tempf /= (2.0f*NIALL_PI);
			sounder->triggerHi(tempf);

			playerBulletCount = 8;
		}

		player->update();

		//Update enemies.
		for(i=0;i<(int)enemies.size();++i)
		{
			if(enemies[i]->update(playerPos))
			{
				tempPos = enemies[i]->getPosition();
				enemyBullets.push_back(shared_ptr<EnemyBullet>(new EnemyBullet(tempPos,
																			   (player->getPosition()-tempPos).normal())));
				collidables.push_back(weak_ptr<Collidable>(enemyBullets.back()));
				tempf = enemyBullets.back()->getAngle();
				tempf += NIALL_PI;
				if(tempf > (2.0f*NIALL_PI))
					tempf -= (2.0f*NIALL_PI);
				else if(tempf < 0.0f)
					tempf += NIALL_PI;
				tempf /= (2.0f*NIALL_PI);
				sounder->triggerPew(tempf);
			}
		}

		//Update bullets, remove any outside the screen.
		for(i=(playerBullets.size()-1);i>=0;--i)
		{
			if(playerBullets[i]->update())
				playerBullets.erase(playerBullets.begin()+i);
		}
		//Update enemy bullets, remove any outside the screen.
		for(i=(enemyBullets.size()-1);i>=0;--i)
		{
			if(enemyBullets[i]->update())
				enemyBullets.erase(enemyBullets.begin()+i);
		}

		//Handle collision detection.
		collisionDetection();

		if(player->getDead())
		{
			const std::vector<shared_ptr<Shard> >& playerShards = player->getShards();

			for(j=0;j<(int)playerShards.size();++j)
				shards.push_back(playerShards[j]);

			if(globalData->getGameCounter().getScore() > globalData->getHighScore())
				globalData->setHighScore(globalData->getGameCounter().getScore());
			globalData->setLastScore(globalData->getGameCounter().getScore());

			eventDisplay.trigger(L"GameOver");
		}

		//Count down for clipboard pickup.
		if((currentPickup == 0) && (pickupClipboardCount > 0))
		{
			--pickupClipboardCount;
			if(!pickupClipboardCount)
				currentPickup = -1;
		}

		//Update points.
		points += 0.001f;
	}

	//Remove any dead enemies.
	for(i=(enemies.size()-1);i>=0;--i)
	{
		if(enemies[i]->getDead())
		{
			const std::vector<shared_ptr<Shard> >& enemyShards = enemies[i]->getShards();

			tempf = origin.getAngle(enemies[i]->getPosition()-playerPos);

			tempf += NIALL_PI;
			if(tempf > (2.0f*NIALL_PI))
				tempf -= (2.0f*NIALL_PI);
			else if(tempf < 0.0f)
				tempf += NIALL_PI;
			tempf /= (2.0f*NIALL_PI);

			sounder->triggerLo(tempf);

			for(j=0;j<(int)enemyShards.size();++j)
				shards.push_back(enemyShards[j]);

			particles.trigger(enemies[i]->getPosition());
			enemies.erase(enemies.begin()+i);

			if(!player->getDead())
				globalData->getGameCounter().incrementCount(L"Enemy Killed", (int)points);

			shakeVal = 1.0f;
		}
	}
	//Remove any dead mines.
	for(i=(mines.size()-1);i>=0;--i)
	{
		if(mines[i]->getDead())
		{
			const std::vector<shared_ptr<Shard> >& mineShards = mines[i]->getShards();

			tempf = origin.getAngle(mines[i]->getPosition()-playerPos);

			tempf += NIALL_PI;
			if(tempf > (2.0f*NIALL_PI))
				tempf -= (2.0f*NIALL_PI);
			else if(tempf < 0.0f)
				tempf += NIALL_PI;
			tempf /= (2.0f*NIALL_PI);

			sounder->triggerMine(tempf);

			for(j=0;j<(int)mineShards.size();++j)
				shards.push_back(mineShards[j]);

			particles.trigger(mines[i]->getPosition());
			mines.erase(mines.begin()+i);
		}
	}
	//Remove any dead bullets.
	for(i=(playerBullets.size()-1);i>=0;--i)
	{
		if(playerBullets[i]->getDead())
			playerBullets.erase(playerBullets.begin()+i);
	}
	//Remove any dead enemy bullets.
	for(i=(enemyBullets.size()-1);i>=0;--i)
	{
		if(enemyBullets[i]->getDead())
			enemyBullets.erase(enemyBullets.begin()+i);
	}
	//Remove any dead pickups.
	for(i=(pickups.size()-1);i>=0;--i)
	{
		if(pickups[i]->getDead())
		{
			currentPickup = pickups[i]->getPickup();
			pickups.erase(pickups.begin()+i);
			pickupState = false;
			sounder->triggerPickup(Random::getGlobal().getRandomFloat());
		}
	}

	//Remove any dead entries from collidables.
	for(i=(collidables.size()-1);i>=0;--i)
	{
		if(collidables[i].expired())
			collidables.erase(collidables.begin()+i);
	}

	for(i=(shards.size()-1);i>=0;--i)
	{
		if(shards[i]->getDead())
			shards.erase(shards.begin()+i);
		else
			shards[i]->update();
	}

	particles.update();
}

//------------------------------------------------------------------------------
void GamePlayState::draw(DrawerPtr d)
{
	float shake;
	unsigned int i;
	TwoFloats tempPos;
	const TwoFloats origin;
	std::wstringstream tempstr;
	const TwoFloats tileSize(d->getDrawingAreaSize()/256.0f);

	if(!backTween0.getFinished())
		backTween0.tick();
	else
	{
		tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
		tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
		backTween0.reset(tempPos);
	}
	if(!backTween1.getFinished())
		backTween1.tick();
	else
	{
		tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
		tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
		backTween1.reset(tempPos);
	}

	if(!backSizeTween0.getFinished())
		backSizeTween0.tick();
	else
		backSizeTween0.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));
	if(!backSizeTween1.getFinished())
		backSizeTween1.tick();
	else
		backSizeTween1.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));

	if(!backColTween.getFinished())
		backColTween.tick();
	else
		backColTween.reset(getBackColour());

	d->fillRect(origin, d->getDrawingAreaSize(), backColour);
	d->drawTiledImage(L"BackgroundTriangles1",
					  origin,
					  d->getDrawingAreaSize(),
					  backPos[0],
					  tileSize*backSize[0],
					  ThreeFloats(1.0f),
					  0.025f);
	d->drawTiledImage(L"BackgroundTriangles2",
					  origin,
					  d->getDrawingAreaSize(),
					  backPos[1],
					  tileSize*backSize[1],
					  ThreeFloats(1.0f),
					  0.025f);

	if(shakeVal > 0.0f)
	{
		shakeVal -= 0.05f;
		shake = sinf(shakeIndex * 2.0f * NIALL_PI) * 8.0f * shakeVal;

		d->translate(TwoFloats(shake, 0.0f));

		shakeIndex += 0.1f;
		if(shakeIndex > 1.0f)
			shakeIndex -= 1.0f;
	}

	for(i=0;i<shards.size();++i)
		shards[i]->draw(d);

	for(i=0;i<mines.size();++i)
		mines[i]->draw(d);

	for(i=0;i<playerBullets.size();++i)
		playerBullets[i]->draw(d);

	for(i=0;i<enemies.size();++i)
		enemies[i]->draw(d);

	for(i=0;i<enemyBullets.size();++i)
		enemyBullets[i]->draw(d);

	for(i=0;i<pickups.size();++i)
		pickups[i]->draw(d);

	if(!player->getDead())
		player->draw(d);

	particles.draw(d);

	tempPos.x = d->getDrawingAreaSize().x - 128.0f;
	tempPos.y = 0.0f;
	d->drawImage(L"PickupBack", tempPos);

	switch(currentPickup)
	{
		case 0:
			{
				ThreeFloats col;
				float tempAlpha = (float)pickupClipboardCount/120.0f;

				if(pickupClipboardCount == 0)
				{
					col = ThreeFloats(1.0f);
					tempAlpha = 1.0f;
				}
				else
				{
					col.x = 1.0f;
					col.y = tempAlpha;
					col.z = tempAlpha;
				}

				
				tempPos.x += (128.0f-38.0f)*0.5f;
				tempPos.y += (128.0f-50.0f)*0.5f;
				d->drawImage(L"PickupClipboard",
							 tempPos,
							 TwoFloats(1.0f),
							 col,
							 tempAlpha);
			}
			break;
		case 1:
			tempPos.x += (128.0f-43.0f)*0.5f;
			tempPos.y += (128.0f-38.0f)*0.5f;
			d->drawImage(L"PickupUndo", tempPos);
			break;
		case 2:
			tempPos.x += (128.0f-40.0f)*0.5f;
			tempPos.y += (128.0f-43.0f)*0.5f;
			d->drawImage(L"PickupSelectAll", tempPos);
			break;
	}

	eventDisplay.draw(d);

	tempPos.x = d->getDrawingAreaSize().x - 192.0f;
	tempPos.y = 140.0f;
	switch(currentPickup)
	{
		case 0:
#ifdef OSX
			d->drawText(L"Cmd-C/Cmd-V",
						L"Font1",
						pickupLayouts[currentPickup],
						tempPos,
						256.0f,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
#else
			d->drawText(L"Ctrl-C/Ctrl-V",
						L"Font1",
						pickupLayouts[currentPickup],
						tempPos,
						256.0f,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
#endif
			break;
		case 1:
#ifdef OSX
			d->drawText(L"Cmd-Z",
						L"Font1",
						pickupLayouts[currentPickup],
						tempPos,
						256.0f,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
#else
			d->drawText(L"Ctrl-Z",
						L"Font1",
						pickupLayouts[currentPickup],
						tempPos,
						256.0f,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
#endif
			break;
		case 2:
#ifdef OSX
			d->drawText(L"Cmd-A/Backspace",
						L"Font1",
						pickupLayouts[currentPickup],
						tempPos,
						256.0f,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
#else
			d->drawText(L"Ctrl-A/Delete",
						L"Font1",
						pickupLayouts[currentPickup],
						tempPos,
						256.0f,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
#endif
			break;
	}

	tempstr << L"Score: " << globalData->getGameCounter().getScore();
	d->drawText(tempstr.str(),
				L"Font0",
				TwoFloats(32.0f, 64.0f),
				512.0f,
				TextLayout::Left,
				1.0f,
				ThreeFloats(1.0f));

	if(player->getDead())
	{
		if(globalData->getHighScore() == globalData->getLastScore())
		{
			d->drawText(L"New High Score!",
						L"Font0",
						textLayout[1],
						TwoFloats(0.0f, 100.0f),
						d->getDrawingAreaSize().x,
						TextLayout::Centre,
						1.0f,
						ThreeFloats(1.0f));
		}

		d->drawText(L"Click to Restart/[Escape] to Title",
					L"Font0",
					textLayout[0],
					TwoFloats(0.0f, d->getDrawingAreaSize().y-40.0f),
					d->getDrawingAreaSize().x,
					TextLayout::Centre,
					1.0f,
					ThreeFloats(1.0f));
	}
}

//------------------------------------------------------------------------------
void GamePlayState::keyDown(const wstring& action,
							const wstring& keyPressed,
							int keyModifier)
{
	int i;
#ifdef OSX
	const int mod = 4;
#else
	const int mod = 1;
#endif

	if(action == L"Escape")
		nextState(L"TitleScreen");
	else if(!player->getDead())
	{
		if(currentPickup == 0)
		{
			if((keyPressed == L"c") && (keyModifier == mod))
			{
				pickupClipboard.clear();

				for(i=0;i<(int)playerBullets.size();++i)
					pickupClipboard.push_back(shared_ptr<PlayerBullet>(new PlayerBullet(*(playerBullets[i]))));

				if(pickupClipboardCount == 0)
					pickupClipboardCount = 120;
			}
			else if((keyPressed == L"v") && (keyModifier == mod))
			{
				for(i=0;i<(int)pickupClipboard.size();++i)
				{
					playerBullets.push_back(shared_ptr<PlayerBullet>(new PlayerBullet(*(pickupClipboard[i]))));
					collidables.push_back(weak_ptr<Collidable>(playerBullets.back()));
				}
			}
		}
		else if((currentPickup == 1) && (keyPressed == L"z") && (keyModifier == mod))
		{
			enemies.clear();
			mines.clear();
			undoClipboardPaste();
			clipboardCopyLast = false;

			currentPickup = -1;
		}
		else if(currentPickup == 2)
		{
			if(!pickupState && (keyPressed == L"a") && (keyModifier == mod))
			{
				pickupState = true;

				for(i=0;i<(int)enemies.size();++i)
					enemies[i]->setBlue();
				for(i=0;i<(int)mines.size();++i)
					mines[i]->setBlue();
			}
			else if(pickupState &&
					((keyPressed == L"Delete") || (keyPressed == L"Backspace")))
			{
				for(i=(enemies.size()-1);i>=0;--i)
				{
					if(enemies[i]->getBlue())
						enemies[i]->setDead();
				}
				for(i=(mines.size()-1);i>=0;--i)
				{
					if(mines[i]->getBlue())
						mines[i]->setDead();
				}

				currentPickup = -1;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState::mouseMove(const TwoFloats& pos)
{
	player->mouseMove(pos);
}

//------------------------------------------------------------------------------
void GamePlayState::mouseDown(int button, const TwoFloats& pos)
{
	if(player->getDead())
		nextState(L"GamePlayState");
}

//------------------------------------------------------------------------------
void GamePlayState::collisionDetection()
{
	float dist;
	unsigned int i, j;
	std::vector<shared_ptr<Collidable> > tempCol;

	for(i=0;i<collidables.size();++i)
	{
		if(!collidables[i].expired())
			tempCol.push_back(shared_ptr<Collidable>(collidables[i]));
	}

	for(i=0;i<tempCol.size();++i)
	{
		//if(!collidables[i].expired())
		{
			shared_ptr<Collidable>& collidableI(tempCol[i]);

			if(collidableI->doNotCount())
				continue;

			for(j=(i+1);j<tempCol.size();++j)
			{
				//if(!collidables[j].expired())
				{
					shared_ptr<Collidable>& collidableJ(tempCol[j]);

					if(collidableJ->doNotCount())
						continue;

					dist = collidableI->getPosition().getDistance(collidableJ->getPosition());

					if(dist < (collidableI->getRadius() + collidableJ->getRadius()))
					{
						collidableI->collided(collidableJ->getName(),
											  collidableJ->getPosition());
						collidableJ->collided(collidableI->getName(),
											  collidableI->getPosition());
					}
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlayState::mainClipboardCopy()
{
	float tempf;
	unsigned int i;

	mainClipboard.enemies.clear();
	mainClipboard.shards.clear();
	mainClipboard.mines.clear();

	for(i=0;i<enemies.size();++i)
	{
		mainClipboard.enemies.push_back(shared_ptr<Enemy>(new Enemy(*(enemies[i]))));
		enemies[i]->setFlash();
	}
	for(i=0;i<mines.size();++i)
	{
		mainClipboard.mines.push_back(shared_ptr<Mine>(new Mine(*(mines[i]))));
		mines[i]->setFlash();
	}
	for(i=0;i<shards.size();++i)
		mainClipboard.shards.push_back(shared_ptr<Shard>(new Shard(*(shards[i]))));

	mainClipboard.playerPos = player->getPosition();

	eventDisplay.trigger(L"Copy");

	tempf = clipboardPitch;
	while((int)tempf == (int)clipboardPitch)
		tempf = Random::getGlobal().getRandomFloat() * 8.0f;
	sounder->triggerSlow(tempf/8.0f);
	clipboardPitch = tempf;
}

//------------------------------------------------------------------------------
void GamePlayState::mainClipboardCut()
{
	int i;
	float tempf;

	mainClipboard.enemies.clear();
	mainClipboard.shards.clear();
	mainClipboard.mines.clear();

	for(i=(enemies.size()-1);i>=0;i-=4)
	{
		mainClipboard.enemies.push_back(shared_ptr<Enemy>(new Enemy(*(enemies[i]))));
		enemies.erase(enemies.begin()+i);
	}
	for(i=(mines.size()-1);i>=0;i-=4)
	{
		mainClipboard.mines.push_back(shared_ptr<Mine>(new Mine(*(mines[i]))));
		mines.erase(mines.begin()+i);
	}
	for(i=(shards.size()-1);i>=0;i-=4)
	{
		mainClipboard.shards.push_back(shared_ptr<Shard>(new Shard(*(shards[i]))));
		shards.erase(shards.begin()+i);
	}

	mainClipboard.playerPos = player->getPosition();

	eventDisplay.trigger(L"Cut");

	tempf = clipboardPitch;
	while((int)tempf == (int)clipboardPitch)
		tempf = Random::getGlobal().getRandomFloat() * 8.0f;
	sounder->triggerSlow(tempf/8.0f);
	clipboardPitch = tempf;
}

//------------------------------------------------------------------------------
void GamePlayState::mainClipboardPaste()
{
	float tempf;
	bool foundEnemy;
	unsigned int i, j;
	TwoFloats tempPos;
	const TwoFloats& playerPos = player->getPosition();
	const TwoFloats incPos = playerPos - mainClipboard.playerPos;
	
	for(i=0;i<(int)mainClipboard.enemies.size();++i)
	{
		if(playerPos.getDistance(mainClipboard.enemies[i]->getPosition()+incPos) > 128.0f)
		{
			enemies.push_back(shared_ptr<Enemy>(new Enemy(*(mainClipboard.enemies[i]))));
			enemies.back()->incPosition(incPos);
			collidables.push_back(weak_ptr<Collidable>(enemies.back()));
		}
	}
	for(i=0;i<(int)mainClipboard.mines.size();++i)
	{
		if(playerPos.getDistance(mainClipboard.mines[i]->getPosition()+incPos) > 128.0f)
		{
			mines.push_back(shared_ptr<Mine>(new Mine(*(mainClipboard.mines[i]))));
			mines.back()->incPosition(incPos);
			collidables.push_back(weak_ptr<Collidable>(mines.back()));
		}
	}
	for(i=0;i<mainClipboard.shards.size();++i)
	{
		foundEnemy = false;
		tempPos = mainClipboard.shards[i]->getPosition();
		for(j=0;j<enemies.size();++j)
		{
			if(tempPos.getDistance(enemies[j]->getPosition()-32.0f) < 16.0f)
			{
				enemies[j]->addShard(shared_ptr<Shard>(new Shard(*(mainClipboard.shards[i]))));

				foundEnemy = true;
				break;
			}
		}

		if(!foundEnemy)
		{
			shards.push_back(shared_ptr<Shard>(new Shard(*(mainClipboard.shards[i]))));
			shards.back()->incPosition(incPos);
		}
	}

	eventDisplay.trigger(L"Paste");

	tempf = clipboardPitch;
	while((int)tempf == (int)clipboardPitch)
		tempf = Random::getGlobal().getRandomFloat() * 8.0f;
	sounder->triggerSlow(tempf/8.0f);
	clipboardPitch = tempf;
}

//------------------------------------------------------------------------------
void GamePlayState::undoClipboardPaste()
{
	float tempf;
	unsigned int i;
	const TwoFloats& playerPos = player->getPosition();
	const TwoFloats incPos = playerPos - undoClipboard.playerPos;

	for(i=0;i<undoClipboard.enemies.size();++i)
	{
		if(playerPos.getDistance(undoClipboard.enemies[i]->getPosition()+incPos) > 128.0f)
		{
			enemies.push_back(shared_ptr<Enemy>(new Enemy(*(undoClipboard.enemies[i]))));
			enemies.back()->incPosition(incPos);
			collidables.push_back(weak_ptr<Collidable>(enemies.back()));
		}
	}
	for(i=0;i<undoClipboard.mines.size();++i)
	{
		if(playerPos.getDistance(undoClipboard.mines[i]->getPosition()+incPos) > 128.0f)
		{
			mines.push_back(shared_ptr<Mine>(new Mine(*(undoClipboard.mines[i]))));
			mines.back()->incPosition(incPos);
			collidables.push_back(weak_ptr<Collidable>(mines.back()));
		}
	}
	for(i=0;i<undoClipboard.shards.size();++i)
	{
		shards.push_back(shared_ptr<Shard>(new Shard(*(undoClipboard.shards[i]))));
		shards.back()->incPosition(incPos);
	}

	eventDisplay.trigger(L"Undo");

	tempf = clipboardPitch;
	while((int)tempf == (int)clipboardPitch)
		tempf = Random::getGlobal().getRandomFloat() * 8.0f;
	sounder->triggerSlow(tempf/8.0f);
	clipboardPitch = tempf;
}

//------------------------------------------------------------------------------
ThreeFloats GamePlayState::getBackColour()
{
	float tempf;
	ThreeFloats retval;

	while((retval == 0.0f) ||
		  (retval == 1.0f) ||
		  (retval == backColour))
	{
		tempf = 0.33f + (Random::getGlobal().getRandomFloat()*0.33f);
		retval.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? tempf : 0.33f;
		tempf = 0.33f + (Random::getGlobal().getRandomFloat()*0.33f);
		retval.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? tempf : 0.33f;
		tempf = 0.33f + (Random::getGlobal().getRandomFloat()*0.33f);
		retval.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? tempf : 0.33f;
	}

	return retval;
}
