//	GamePlayState.h - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef GAMEPLAYSTATE_H_
#define GAMEPLAYSTATE_H_

#include "drawing/TextLayout.h"
#include "ParticleGenerator.h"
#include "EventDisplay.h"
#include "GameState.h"
#include "Tweener.h"

#include <vector>

class PastarenaGlobalData;
class PastarenaSounder;
class PlayerBullet;
class EnemyBullet;
class Collidable;
class Player;
class Pickup;
class Enemy;
class Shard;
class Mine;

///	The main gameplay state.
class GamePlayState : public GameState
{
  public:
	///	Constructor.
	GamePlayState(Application& app, SounderPtr sound);
	///	Destructor.
	~GamePlayState();

	///	Called frequently to update the state.
	void update();
	///	Called to draw the state.
	void draw(DrawerPtr d);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when the mouse is moved.
	void mouseMove(const TwoFloats& pos);
	///	Used to restart when the player dies.
	void mouseDown(int button, const TwoFloats& pos);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"GamePlayState";};
	///	Used to construct instances of this state.
	static GameStateCreator<GamePlayState> creator;
  private:
	///	Helper method. Handles collisions.
	void collisionDetection();
	///	Helper method. Conducts a clipboard copy.
	void mainClipboardCopy();
	///	Helper method. Conducts a clipboard cut.
	void mainClipboardCut();
	///	Helper method. Conducts a clipboard paste.
	void mainClipboardPaste();
	///	Helper method. Conducts a clipboard paste.
	void undoClipboardPaste();

	///	Helper method. Calculates a reasonable background colour.
	ThreeFloats getBackColour();

	///	Main game clipboard structure.
	struct MainClipboard
	{
		///	The enemies stored in the clipboard.
		std::vector<shared_ptr<Enemy> > enemies;
		///	The mines stored in the clipboard.
		std::vector<shared_ptr<Mine> > mines;
		///	The shards stored in the clipboard.
		std::vector<shared_ptr<Shard> > shards;

		///	The position of the player when the most recent copy was taken.
		TwoFloats playerPos;
	};

	///	Positions of the two background textures.
	TwoFloats backPos[2];
	///	Tweener for the first background texture.
	STweener<TwoFloats> backTween0;
	///	Tweener for the second background texture.
	STweener<TwoFloats> backTween1;

	///	Sizes of the two background textures.
	TwoFloats(backSize[2]);
	///	Tweener for the firstfirst background texture.
	STweener<TwoFloats> backSizeTween0;
	///	Tweener for the second background texture.
	STweener<TwoFloats> backSizeTween1;

	///	The background colour.
	ThreeFloats backColour;
	///	The tweener for the background colour.
	STweener<ThreeFloats> backColTween;

	///	The 'click to title' TextLayout.
	TextLayout textLayout[2];
	///	The pickup TextLayouts.
	TextLayout pickupLayouts[3];

	///	The player.
	shared_ptr<Player> player;
	///	The player's bullets.
	std::vector<shared_ptr<PlayerBullet> > playerBullets;
	///	Used to determine when to add new bullets.
	int playerBulletCount;

	///	The active enemies.
	std::vector<shared_ptr<Enemy> > enemies;
	///	The enemy bullets.
	std::vector<shared_ptr<EnemyBullet> > enemyBullets;
	///	The active mines.
	std::vector<shared_ptr<Mine> > mines;
	///	Any un-parented shards in the game.
	std::vector<shared_ptr<Shard> > shards;
	///	Any pickups in the game.
	std::vector<shared_ptr<Pickup> > pickups;

	///	Used for collision detection.
	std::vector<weak_ptr<Collidable> > collidables;

	///	The main clipboard.
	MainClipboard mainClipboard;
	///	The undo clipboard.
	MainClipboard undoClipboard;
	///	Used to determine when to do a clipboard action.
	int clipboardCount;
	///	True if the last clipboard action was a copy.
	bool clipboardCopyLast;
	///	Used to ensure we don't get the same pitches with every event.
	float clipboardPitch;

	///	Used to display which clipboard events have happened.
	EventDisplay eventDisplay;

	///	Used to determine when to add a pickup.
	int pickupCount;
	///	The currently-selected pickup.
	int currentPickup;
	///	The state of the currently-selected pickup.
	bool pickupState;
	///	The clipboard for the clipboard pickup.
	std::vector<shared_ptr<PlayerBullet> > pickupClipboard;
	///	The count down for the clipboard pickup.
	int pickupClipboardCount;

	///	For the screen shake.
	float shakeIndex;
	///	For the screen shake.
	float shakeVal;
	///	For when something dies.
	ParticleGenerator particles;

	///	Used to give the player more points for kills the further into the game they go.
	float points;

	///	Our sounder.
	shared_ptr<PastarenaSounder> sounder;
	///	Our global data.
	shared_ptr<PastarenaGlobalData> globalData;
};

#endif
