//	PastarenaSounder.cpp - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PastarenaSounder.h"
#include "Random.h"

#include "PastaPickup.inl"
#include "PastaSlow.inl"
#include "PastaMine.inl"
#include "PastaPew.inl"
#include "PastaLo.inl"
#include "PastaHi.inl"

#include <cassert>
#include <sstream>
#include <string>

//	----------------------------------------------------------------------------
PastarenaSounder::PastarenaSounder()
{
	hiSynth.loadParameters(PastaHi::numParameters, PastaHi::parameters);
	hiDelay.setDelay(0.125f);
	hiDelay.setFeedback(0.75f);

	loSynth.loadParameters(PastaLo::numParameters, PastaLo::parameters);
	loDelay.setDelay(0.25f);
	loDelay.setFeedback(0.75f);

	slowSynth.loadParameters(PastaSlow::numParameters, PastaSlow::parameters);
	slowDelay.setDelay(0.33f);
	slowDelay.setFeedback(0.75f);

	mineSynth.loadParameters(PastaMine::numParameters, PastaMine::parameters);
	mineDelay.setDelay(0.167f);
	mineDelay.setFeedback(0.75f);

	pickupSynth.loadParameters(PastaPickup::numParameters, PastaPickup::parameters);
	pickupDelay.setDelay(0.167f);
	pickupDelay.setFeedback(0.75f);

	pewSynth.loadParameters(PastaPew::numParameters, PastaPew::parameters);
	pewDelay.setDelay(0.125f);
	pewDelay.setFeedback(0.75f);
}

//	----------------------------------------------------------------------------
PastarenaSounder::~PastarenaSounder()
{

}

//	----------------------------------------------------------------------------
void PastarenaSounder::getAudio(float *input, float *output, int numSamples)
{
	int i;

	assert(numSamples <= (BufferSize/2));

	for(i=0;i<(numSamples*2);++i)
	{
		loBuffer[i] = 0.0f;
		slowBuffer[i] = 0.0f;
		mineBuffer[i] = 0.0f;
		pickupBuffer[i] = 0.0f;
		pewBuffer[i] = 0.0f;
	}

	hiSynth.getAudio(output, numSamples);
	hiDelay.getAudio(output, numSamples);

	loSynth.getAudio(loBuffer, numSamples);
	loDelay.getAudio(loBuffer, numSamples);

	slowSynth.getAudio(slowBuffer, numSamples);
	slowDelay.getAudio(slowBuffer, numSamples);

	mineSynth.getAudio(mineBuffer, numSamples);
	mineDelay.getAudio(mineBuffer, numSamples);

	pickupSynth.getAudio(pickupBuffer, numSamples);
	pickupDelay.getAudio(pickupBuffer, numSamples);

	pewSynth.getAudio(pewBuffer, numSamples);
	pewDelay.getAudio(pewBuffer, numSamples);

	for(i=0;i<(numSamples*2);++i)
	{
		output[i] += loBuffer[i];
		output[i] += slowBuffer[i];
		output[i] += mineBuffer[i];
		output[i] += pickupBuffer[i];
		output[i] += pewBuffer[i];
	}
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerHi(float val)
{
	int index = (int)(val * 7.99f);

	hiSynth.triggerNote(pitches[index]*2.0f, 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerLo(float val)
{
	int index = (int)(val * 7.99f);

	loSynth.triggerNote(pitches[index], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerSlow(float val)
{
	int index = (int)(val * 7.99f);

	slowSynth.triggerNote(pitches[index], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerMine(float val)
{
	int index = (int)(val * 7.99f);

	mineSynth.triggerNote(pitches[index], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerPickup(float val)
{
	int index = (int)(val * 7.99f);

	pickupSynth.triggerNote(pitches[index], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::triggerPew(float val)
{
	int index = (int)(val * 7.99f);

	pewSynth.triggerNote(pitches[index], 0);
}

//	----------------------------------------------------------------------------
void PastarenaSounder::loadPitches()
{
	int index;
	FilePath pitchFile;
	FilePath pitchesDir = FilePath::getDataFilesDir().getChild(L"pitches");

	index = Random::getGlobal().getRandomInt(pitchesDir.getNumChildren());
	pitchFile = pitchesDir.getChild(index);

	if(pitchFile.exists())
	{
		if(pitchFile.getExtension() == L"txt")
		{
			UTF8File phil(pitchFile);

			if(!phil.getFail())
			{
				int i;
				float tempf;
				std::wstring tempstr;
				std::wstringstream tempstr2;

				i = 0;
				while(!phil.readLine(tempstr))
				{
					assert(i < 8);

					tempstr2.clear();
					tempstr2.str(L"");
					tempstr2 << tempstr;

					tempstr2 >> tempf;

					pitches[i] = tempf * 0.25f;

					++i;
				}
			}
		}
	}
}
