//	Shard.cpp - Represents a shard (of an enemy, the player, etc.).
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "Random.h"
#include "Shard.h"

//------------------------------------------------------------------------------
Shard::Shard(const std::wstring& im,
			 const ThreeFloats& col,
			 const TwoFloats& relPos):
image(im),
colour(col),
colourTweener(colour, colour, 0.04f),
relativePosition(relPos),
directionTweener(direction, TwoFloats(0.0f, 1.0f), 0.01f),
rotation(0.0f),
alpha(1.0f),
alphaTweener(alpha, 0.0f, 0.01f)
{
	colourTweener.setIndex(1.0f);

	rotSpeed = Random::getGlobal().getRandomFloat() > 0.5f ? -0.1f : 0.1f;

	directionTweener.setDelay(Random::getGlobal().getRandomFloat());
	directionTweener.setDelta(0.01f + (Random::getGlobal().getRandomFloat() * 0.04f));
	alphaTweener.setDelay(Random::getGlobal().getRandomFloat() * 0.5f);
	alphaTweener.setDelta(0.005f + (Random::getGlobal().getRandomFloat() * 0.015f));
}

//------------------------------------------------------------------------------
Shard::Shard(const Shard& other):
image(other.image),
colour(other.colour),
colourTweener(colour, colour, 0.05f),
position(other.position),
relativePosition(other.relativePosition),
direction(other.direction),
directionTweener(direction, TwoFloats(0.0f, 1.0f), 0.01f),
rotation(other.rotation),
rotSpeed(other.rotSpeed),
alpha(other.alpha),
alphaTweener(alpha, 0.0f, 0.01f)
{
	colourTweener.setIndex(1.0f);

	directionTweener.setDelay(Random::getGlobal().getRandomFloat());
	directionTweener.setDelta(0.01f + (Random::getGlobal().getRandomFloat() * 0.04f));
	alphaTweener.setDelay(Random::getGlobal().getRandomFloat() * 0.5f);
	alphaTweener.setDelta(0.005f + (Random::getGlobal().getRandomFloat() * 0.015f));
}

//------------------------------------------------------------------------------
Shard::~Shard()
{

}

//------------------------------------------------------------------------------
void Shard::update()
{
	if(!directionTweener.getFinished())
		directionTweener.tick();
	if(!alphaTweener.getFinished())
		alphaTweener.tick();
	
	rotation += rotSpeed;
	position += direction * 2.0f;
}

//------------------------------------------------------------------------------
void Shard::draw(shared_ptr<Drawer>& d, float fadeIn)
{
	const Drawer::Image * const im = d->getInternalImageData(image);
	const TwoFloats pivot = im->size * 0.5f;

	if(!colourTweener.getFinished())
		colourTweener.tick();

	d->drawRotImage(image,
					position,
					rotation,
					pivot,
					TwoFloats(1.0f),
					colour,
					alpha*fadeIn);
}

//------------------------------------------------------------------------------
void Shard::setPosition(const TwoFloats& pos)
{
	position = pos + relativePosition;
}

//------------------------------------------------------------------------------
void Shard::setDirection(const TwoFloats& dir)
{
	direction = dir;
	directionTweener.setIndex(0.0f);
	directionTweener.reset(TwoFloats(0.0f, 1.0f));
}

//------------------------------------------------------------------------------
void Shard::incPosition(const TwoFloats& pos)
{
	position += pos;
}

//------------------------------------------------------------------------------
void Shard::reset(const TwoFloats& relPos)
{
	relativePosition = relPos;
	alpha = 1.0f;
	alphaTweener.setIndex(0.0f);
}

//------------------------------------------------------------------------------
void Shard::setFlash()
{
	ThreeFloats tempCol(colour);

	colour = ThreeFloats(1.0f);
	colourTweener.reset(tempCol);
}

//------------------------------------------------------------------------------
void Shard::setBlue()
{
	if(colour.x > 0.25f)
		colour.x -= 0.25f;
	if(colour.y > 0.25f)
		colour.y -= 0.25f;
	colour.z = 1.0f;
}
