//	Shard.h - Represents a shard (of an enemy, the player, etc.).
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SHARD_H_
#define SHARD_H_

#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <string>

class Drawer;

///	Represents a shard (of an enemy, the player, etc.).
class Shard
{
  public:
	///	Constructor.
	/*!
		\param im The image this shard uses.
		\param col The colour of the shard.
		\param relPos The shard's position relative to the value passed in
		to setPosition().
	 */
	Shard(const std::wstring& im,
		  const ThreeFloats& col,
		  const TwoFloats& relPos);
	///	Copy Constructor.
	Shard(const Shard& other);
	///	Destructor.
	~Shard();

	///	Moves the shard.
	void update();
	///	Draws the shard.
	void draw(shared_ptr<Drawer>& d, float fadeIn = 1.0f);

	///	Sets the shard's position.
	void setPosition(const TwoFloats& pos);
	///	Sets the shard's direction.
	void setDirection(const TwoFloats& dir);

	///	Used to set the shard's starting position on paste.
	void incPosition(const TwoFloats& pos);
	///	Resets the shard.
	void reset(const TwoFloats& relPos);

	///	Flashes the shard white.
	void setFlash();
	///	Marks the shard blue.
	void setBlue();

	///	Returns the shard's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the shard's alpha.
	float getAlpha() const {return alpha;};
	///	Returns true if the shard is dead.
	bool getDead() const {return (alpha <= 0.0f);};
  private:
	///	The shard's image.
	std::wstring image;
	///	The shard's colour.
	ThreeFloats colour;
	///	The shard's colour tweener.
	Tweener<ThreeFloats> colourTweener;
	///	The shard's position.
	TwoFloats position;
	///	The shard's relative position.
	TwoFloats relativePosition;
	///	The shard's direction.
	TwoFloats direction;
	///	The shard's direction tweener.
	LogTweener<TwoFloats> directionTweener;
	///	The shard's rotation.
	float rotation;
	///	Used to rotate the shard.
	float rotSpeed;
	///	The shard's alpha value.
	float alpha;
	///	The shard's alpha tweener.
	LogTweener<float> alphaTweener;
};

#endif
