//	TitleScreen.cpp - The title screen state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PastarenaGlobalData.h"
#include "drawing/Drawer.h"
#include "TitleScreen.h"
#include "Random.h"

#include <sstream>

//------------------------------------------------------------------------------
GameStateCreator<TitleScreen> TitleScreen::creator;

//------------------------------------------------------------------------------
TitleScreen::TitleScreen(Application& app, SounderPtr sound):
GameState(app, sound),
backTween0(backPos[0], backPos[0], 0.001f),
backTween1(backPos[1], backPos[1], 0.0015f),
backSizeTween0(backSize[0], backSize[0], 0.002f),
backSizeTween1(backSize[1], backSize[1], 0.0011f),
backColour(0.33f, 0.5f, 0.33f),
backColTween(backColour, backColour, 0.001f),
fadeIn(1.0f)
{
	TwoFloats tempPos;

	globalData = dynamic_pointer_cast<PastarenaGlobalData>(getGlobalData());

	//Setup the background.
	tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
	tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
	backTween0.reset(tempPos);

	tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
	tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
	backPos[1] = TwoFloats(0.33f, 0.1f);
	backTween1.reset(tempPos);

	backSize[0] = 1.0f;
	backSize[1] = 1.0f;
	backSizeTween0.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));
	backSizeTween1.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));

	backColour = getBackColour();
	backColTween.reset(getBackColour());
}

//------------------------------------------------------------------------------
TitleScreen::~TitleScreen()
{

}

//------------------------------------------------------------------------------
void TitleScreen::draw(DrawerPtr d)
{
	TwoFloats tempPos;
	const TwoFloats origin;
	std::wstringstream tempstr;
	const TwoFloats tileSize(d->getDrawingAreaSize()/256.0f);

	if(!backTween0.getFinished())
		backTween0.tick();
	else
	{
		tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
		tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
		backTween0.reset(tempPos);
	}
	if(!backTween1.getFinished())
		backTween1.tick();
	else
	{
		tempPos.x = Random::getGlobal().getRandomFloat() * 6.0f;
		tempPos.y = Random::getGlobal().getRandomFloat() * 6.0f;
		backTween1.reset(tempPos);
	}

	if(!backSizeTween0.getFinished())
		backSizeTween0.tick();
	else
		backSizeTween0.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));
	if(!backSizeTween1.getFinished())
		backSizeTween1.tick();
	else
		backSizeTween1.reset(TwoFloats(0.5f+(Random::getGlobal().getRandomFloat()*0.5f)));

	if(!backColTween.getFinished())
		backColTween.tick();
	else
		backColTween.reset(getBackColour());

	d->fillRect(origin, d->getDrawingAreaSize(), backColour);
	d->drawTiledImage(L"BackgroundTriangles1",
					  origin,
					  d->getDrawingAreaSize(),
					  backPos[0],
					  tileSize*backSize[0],
					  ThreeFloats(1.0f),
					  0.025f);
	d->drawTiledImage(L"BackgroundTriangles2",
					  origin,
					  d->getDrawingAreaSize(),
					  backPos[1],
					  tileSize*backSize[1],
					  ThreeFloats(1.0f),
					  0.025f);

	tempPos.x = (d->getDrawingAreaSize().x-850.0f) * 0.5f;
	tempPos.y = 32.0f;
	d->drawImage(L"Title", tempPos);

	d->drawText(L"Scores:",
				L"Font0",
				textLayouts[2],
				TwoFloats(0.0f, 340.0f),
				d->getDrawingAreaSize().x,
				TextLayout::Centre,
				1.0f,
				ThreeFloats(1.0f));

	tempstr << L"Hi: " << globalData->getHighScore();
	tempstr << L"\nLast: " << globalData->getLastScore();
	d->drawText(tempstr.str(),
				L"Font1",
				textLayouts[0],
				TwoFloats(0.0f, 400.0f),
				d->getDrawingAreaSize().x,
				TextLayout::Centre,
				1.0f,
				ThreeFloats(1.0f));

	d->drawText(L"Click to Start/[Escape] to Quit",
				L"Font0",
				textLayouts[1],
				TwoFloats(0.0f, d->getDrawingAreaSize().y-40.0f),
				d->getDrawingAreaSize().x,
				TextLayout::Centre,
				1.0f,
				ThreeFloats(1.0f));

	if(fadeIn > 0.0f)
	{
		d->fillRect(origin,
					d->getDrawingAreaSize(),
					ThreeFloats(0.33f, 0.5f, 0.66f),
					fadeIn);
		fadeIn -= 0.02f;
	}
}

//------------------------------------------------------------------------------
void TitleScreen::keyDown(const std::wstring& action,
						  const std::wstring& keyPressed,
						  int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void TitleScreen::mouseDown(int button, const TwoFloats& pos)
{
	nextState(L"GamePlayState");
}

//------------------------------------------------------------------------------
ThreeFloats TitleScreen::getBackColour()
{
	float tempf;
	ThreeFloats retval;

	while((retval == 0.0f) ||
		  (retval == 1.0f) ||
		  (retval == backColour))
	{
		tempf = 0.33f + (Random::getGlobal().getRandomFloat()*0.33f);
		retval.x = (Random::getGlobal().getRandomFloat() > 0.5f) ? tempf : 0.33f;
		tempf = 0.33f + (Random::getGlobal().getRandomFloat()*0.33f);
		retval.y = (Random::getGlobal().getRandomFloat() > 0.5f) ? tempf : 0.33f;
		tempf = 0.33f + (Random::getGlobal().getRandomFloat()*0.33f);
		retval.z = (Random::getGlobal().getRandomFloat() > 0.5f) ? tempf : 0.33f;
	}

	return retval;
}
