//	SimpleSynth.cpp - Simple analogue-esque synth.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SimpleSynth.h"
#include "MiscHelper.h"

#include <cmath>

using namespace std;

//------------------------------------------------------------------------------
SimpleSynth::SimpleSynth():
MonophonicSynth(),
waveAmount(0.0f),
detunedSawAmount(0.0f),
filterCutoff(0.5f),
filterRes(0.0f),
filterAttackCurve(false),
filterReleaseCurve(false),
filterAttackTime(0.25f),
filterReleaseTime(0.5f),
filterEnvAmount(0.01f),
attackCurve(false),
releaseCurve(false),
attackTime(0.25f),
releaseTime(0.5f),
noteLength(0.5f),
distGain(1.0f),
octave(1.0f),
level(1.0f),
tableIndex(0.0f),
detuneIndex(0.0f),
frequency(440.0f),
detuneFreq(439.0f),
noteIndex(0.0f),
increment(1.0f/44100.0f),
noteWaiting(false),
waitingSample(0)
{
	tableSize = WavetableSingleton::getInstance().getTableSize();
	/*sawTable = WavetableSingleton::getInstance().getSawTable();
	squareTable = WavetableSingleton::getInstance().getSquareTable();*/
}

//------------------------------------------------------------------------------
SimpleSynth::~SimpleSynth()
{
	
}

//------------------------------------------------------------------------------
void SimpleSynth::getAudio(float *buffer, int numSamples)
{
	int i;
	float out;
	float saw;
	float saw2;
	float tempf;
	int tempint;
	float square;
	float ampEnv;
	float filtEnv;
	const float tableInc = (float)tableSize/(44100.0f/frequency);
	const float detuneInc = (float)tableSize/(44100.0f/detuneFreq);
	const float actualNoteLength = noteLength * 2.0f;
	const float attackLength = attackTime * (actualNoteLength*0.999f);
	const float attackLengthInv = 1.0f/attackLength;
	const float releaseLength = releaseTime * (actualNoteLength-attackLength);
	const float releaseLengthInv = 1.0f/releaseLength;
	const float holdEnd = attackLength + (actualNoteLength - (attackLength+releaseLength));
	const float filtAttack = filterAttackTime * (actualNoteLength*0.999f);
	const float filtAttackInv = 1.0f/filtAttack;
	const float filtRelease = filterReleaseTime * (actualNoteLength-filtAttack);
	const float filtReleaseInv = 1.0f/filtRelease;
	const float filtHold = filtAttack + (actualNoteLength - (filtAttack+filtRelease));

	if(!getActive())
		return;

	for(i=0;i<numSamples;++i)
	{
		if(noteWaiting && (waitingSample >= 0))
		{
			--waitingSample;

			if(waitingSample <= 0)
				noteWaiting = false;
			else
				continue;
		}
		//Increment our position within the note, quit if we've gone past
		//noteLength.
		noteIndex += increment;
		if(noteIndex > actualNoteLength)
		{
			setActive(false);
			break;
		}

		//Calculate the amplitude envelope.
		if(noteIndex < attackLength)
		{
			if(attackCurve)
				ampEnv =  fakeExp(noteIndex*attackLengthInv);
			else
				ampEnv =  fakeLog(noteIndex*attackLengthInv);
		}
		else if(noteIndex < holdEnd)
			ampEnv = 1.0f;
		else
		{
			if(releaseCurve)
				ampEnv = 1.0f-fakeLog((noteIndex-holdEnd)*releaseLengthInv);
			else
				ampEnv = 1.0f-fakeExp((noteIndex-holdEnd)*releaseLengthInv);
		}

		//Calculate the filter envelope.
		if(noteIndex < filtAttack)
		{
			if(filterAttackCurve)
				filtEnv =  fakeExp(noteIndex*filtAttackInv) * filterEnvAmount;
			else
				filtEnv =  fakeLog(noteIndex*filtAttackInv) * filterEnvAmount;
		}
		else if(noteIndex < filtHold)
			filtEnv = filterEnvAmount;
		else
		{
			if(filterReleaseCurve)
				filtEnv = (1.0f-fakeLog((noteIndex-filtHold)*filtReleaseInv)) * filterEnvAmount;
			else
				filtEnv = (1.0f-fakeExp((noteIndex-filtHold)*filtReleaseInv)) * filterEnvAmount;
		}

		//Generate the sawtooth audio.
		tempint = (int)tableIndex;
		tempf = tableIndex - (float)tempint;
		saw = sawTable[tempint] * (1.0f-tempf);
		if(tempint < (tableSize-1))
			saw += sawTable[tempint+1] * tempf;
		else
			saw += sawTable[0] * tempf;

		//Generate the square wave audio.
		tempint = (int)tableIndex;
		tempf = tableIndex - (float)tempint;
		square = squareTable[tempint] * (1.0f-tempf);
		if(tempint < (tableSize-1))
			square += squareTable[tempint+1] * tempf;
		else
			square += squareTable[0] * tempf;

		//Mix the 2 waveforms together.
		out = saw * (1.0f-waveAmount);
		out += square * waveAmount;

		//Add the detuned sawtooth.
		if(detunedSawAmount > 0.0f)
		{
			tempint = (int)detuneIndex;
			tempf = detuneIndex - (float)tempint;
			saw2 = sawTable[tempint] * (1.0f-tempf);
			if(tempint < (tableSize-1))
				saw2 += sawTable[tempint+1] * tempf;
			else
				saw2 += sawTable[0] * tempf;

			out += saw2 * detunedSawAmount;
		}

		//Update the table index.
		tableIndex += tableInc;
		if(tableIndex > (float)tableSize)
			tableIndex -= (float)tableSize;

		//Update the detuned sawtooth index.
		detuneIndex += detuneInc;
		if(detuneIndex > (float)tableSize)
			detuneIndex -= (float)tableSize;

		//Update the filter's parameters.
		tempf = filterCutoff * (11025.0f-50.0f);
		if(filtEnv < 0.0f)
			tempf += filtEnv * tempf;
		else
			tempf += filtEnv * (11025.0f-tempf);
		filter.setCutoffAndRes(50.0f + tempf, filterRes);

		//Apply the filter to the waves.
		out = filter.process(out);

		//Apply the distortion.
		out = tanhf(out * distGain);

		//Apply the amplitude envelope to the output.
		out *= ampEnv * 0.125f * level;

		buffer[i*2] += out;
		buffer[(i*2)+1] += out;
	}
}

//------------------------------------------------------------------------------
void SimpleSynth::triggerNote(float freq, int sample)
{
	frequency = freq * octave;
	detuneFreq = frequency * 0.999f;

	sawTable = WavetableSingleton::getInstance().getSawTable(frequency);
	squareTable = WavetableSingleton::getInstance().getSquareTable(frequency);

	noteWaiting = true;
	waitingSample = sample;

	//tableIndex = 0.0f;
	noteIndex = 0.0f;
	setActive(true);

}

//------------------------------------------------------------------------------
void SimpleSynth::setParameter(const string& name, float value, bool direct)
{
	if(name == "waveAmount")
		waveAmount = value;
	else if(name == "detunedSawAmount")
		detunedSawAmount = value;
	else if(name == "filterCutoff")
	{
		if(direct)
			filterCutoff = value;
		else
			filterCutoff = (fakeExp(value) * 0.5f) + (value * 0.5f);
	}
	else if(name == "filterRes")
		filterRes = value;
	else if(name == "filterAttackCurve")
		filterAttackCurve = (value > 0.5f);
	else if(name == "filterReleaseCurve")
		filterReleaseCurve = (value > 0.5f);
	else if(name == "filterAttackTime")
		filterAttackTime = value;
	else if(name == "filterReleaseTime")
		filterReleaseTime = value;
	else if(name == "filterEnvAmount")
		filterEnvAmount = value;
	else if(name == "attackCurve")
		attackCurve = (value > 0.5f);
	else if(name == "releaseCurve")
		releaseCurve = (value > 0.5f);
	else if(name == "attackTime")
		attackTime = value;
	else if(name == "releaseTime")
		releaseTime = value;
	else if(name == "noteLength")
	{
		if(direct)
			noteLength = value;
		else
			noteLength = 0.05f + (value*0.95f);
	}
	else if(name == "distGain")
	{
		if(direct)
			distGain = value;
		else
			distGain = 1.0f + (value * 3.0f);
	}
	else if(name == "octave")
	{
		if(direct)
			octave = value;
		else
		{
			if(value < 0.9f)
				octave = 4.0f;
			else if(value < 1.9f)
				octave = 2.0f;
			else if(value < 2.9f)
				octave = 1.0f;
			else if(value < 3.9f)
				octave = 0.5f;
			else
				octave = 0.25f;
		}
	}
	else if(name == "level")
		level = value;
}

//------------------------------------------------------------------------------
float SimpleSynth::getParameter(int i) const
{
	switch(i)
	{
		case 0:
			return waveAmount;
		case 1:
			return filterCutoff;
		case 2:
			return filterRes;
		case 3:
			return filterAttackCurve;
		case 4:
			return filterReleaseCurve;
		case 5:
			return filterAttackTime;
		case 6:
			return filterReleaseTime;
		case 7:
			return filterEnvAmount;
		case 8:
			return attackCurve;
		case 9:
			return releaseCurve;
		case 10:
			return attackTime;
		case 11:
			return releaseTime;
		case 12:
			return noteLength;
		case 13:
			return distGain;
		case 14:
			return octave;
		case 15:
			return detunedSawAmount;
		case 16:
			return level;
		default:
			return 0.0f;
	}
}

//------------------------------------------------------------------------------
string SimpleSynth::getParameterName(int i) const
{
	switch(i)
	{
		case 0:
			return "waveAmount";
		case 1:
			return "filterCutoff";
		case 2:
			return "filterRes";
		case 3:
			return "filterAttackCurve";
		case 4:
			return "filterReleaseCurve";
		case 5:
			return "filterAttackTime";
		case 6:
			return "filterReleaseTime";
		case 7:
			return "filterEnvAmount";
		case 8:
			return "attackCurve";
		case 9:
			return "releaseCurve";
		case 10:
			return "attackTime";
		case 11:
			return "releaseTime";
		case 12:
			return "noteLength";
		case 13:
			return "distGain";
		case 14:
			return "octave";
		case 15:
			return "detunedSawAmount";
		case 16:
			return "level";
		default:
			return "";
	}
}

//------------------------------------------------------------------------------
SimpleSynth::WavetableSingleton& SimpleSynth::WavetableSingleton::getInstance()
{
	static WavetableSingleton retval;

	return retval;
}
